/**
 * The two pre-existing link menu items are context-openlink (label = "&openLinkCmd.label;" = "Open Link in New Window",
 *														oncommand="gContextMenu.openLink();")
 * and context-openlinkintab (label="&openLinkCmdInTab.label;" = "Open Link in New Tab", oncommand="gContextMenu.openLinkInTab();"), the oncommand
 * functions being the following (located in nsContextMenu.js):
 *  openLink : function () { openNewWindowWith(this.linkURL, this.target.ownerDocument, null, false); }
 *  openLinkInTab: function() { openNewTabWith(this.linkURL, this.target.ownerDocument, null, null, false); }
 *
 * The browser automatically shows both of these menu items, and we add others, including the opposite of context-openlinkintab.
 *
 * Note that we do not provide a menu option for the /default/ action of clicking on a link, namely, obey HTML
 * window targets. (We do implement an option for opening a link in the current tab/window using loadURI, a low-level function
 * which always seems to open a link thus.)
 *
 * We also provide the option of moving all the open link items into a submenu, to reduce clutter for those who like
 * ultra-compact menus.
 *
 * Finally, we provide similar submenus for images and background images, while removing the default menu item for
 * 'view image' and 'view background image' (since if we are providing a submenu, we might as well put everything into it).
 *
 * Changes in this major version (1.5) from version 1.4 are mainly contained in the openlinkOpen*With functions, which are based on
 * openNewTabWith and openNewWindowWith in utilityOverlay.js in browser.jar.  (Prior to Firefox 3 these latter functions
 * were located in contentAreaUtils.js in toolkit.jar, and had slightly different parameters and implementation.)  Additionally,
 * the openlinkOpen*ImageIn* functions (which used to call the openlinkOpen*With functions directly) now call the new
 * openlinkViewImageWith function which is based on the function viewImage in nsContextMenu.js in browser.jar, taking care of
 * improved security checks.
 *
 * The change in this minor version (1.5.1) from version 1.5 consists of a bugfix for broken background window functionality.
 *
 **/

var gOpenlinkOpenLinkMenuItems = new Array('context-openlinkintab',
                                           'openlink-openlinkinbackgroundtab',
                                           'openlink-openlinkinforegroundtab',
                                           'context-openlink',
                                           'openlink-openlinkinbackgroundwindow',
                                           'openlink-openlinkhere'
                                          );

var gOpenlinkOpenLinkMenuMenuItems = new Array('openlink-openlinkinnewtabmenu',
                                               'openlink-openlinkinbackgroundtabmenu',
                                               'openlink-openlinkinforegroundtabmenu',
                                               'openlink-openlinkinnewwindowmenu',
                                               'openlink-openlinkinbackgroundwindowmenu',
                                               'openlink-openlinkheremenu'
                                              );

var gOpenlinkViewImageMenuItems = new Array('openlink-viewimageinnewtab',
                                            'openlink-viewimageinbackgroundtab',
                                            'openlink-viewimageinforegroundtab',
                                            'openlink-viewimageinnewwindow',
                                            'openlink-viewimageinbackgroundwindow',
                                            'openlink-viewimagehere'
                                           );


var gOpenlinkViewBackgroundImageMenuItems = new Array('openlink-viewbackgroundimageinnewtab',
                                                      'openlink-viewbackgroundimageinbackgroundtab',
                                                      'openlink-viewbackgroundimageinforegroundtab',
                                                      'openlink-viewbackgroundimageinnewwindow',
                                                      'openlink-viewbackgroundimageinbackgroundwindow',
                                                      'openlink-viewbackgroundimagehere'
                                                     );
var gCount;
const gMAX = 50;
var gCurrWindow;


window.addEventListener('load', openlinkInit, false);


/**
 * Registers a listener so that we can specify a function to be called when the context area menu or the
 * view image list menus pop up.
 */
function openlinkInit() {
	var menu = document.getElementById('contentAreaContextMenu');
	menu.addEventListener("popupshowing", openlinkShowContentAreaContextMenuItemsOnSuitableElements, false);
	menu = document.getElementById('openlink-openlinkin');
	menu.addEventListener("popupshowing", openlinkShowOpenLinkContextMenuItems, false);
	menu = document.getElementById('openlink-viewimage');
	menu.addEventListener("popupshowing", openlinkShowViewImageContextMenuItems, false);
	menu = document.getElementById('openlink-viewbackgroundimage');
	menu.addEventListener("popupshowing", openlinkShowViewBackgroundImageContextMenuItems, false);
}


/**
 * This function is called when the context area menu pops up.
 * It decides which open link menu elements should be shown.
 */
function openlinkShowContentAreaContextMenuItemsOnSuitableElements() {
	var tabsOpenInBg = window.getBoolPref("browser.tabs.loadInBackground", false);
	//If the page context menu is open:
	if (gContextMenu) {
		//Decide if user is on an openable link:
		var isOpenableLink = ( gContextMenu.onSaveableLink ||
		                                          ( gContextMenu.inDirList && gContextMenu.onLink ) );
		//Decide if user wants link items instead of submenu:
		var prefs = Components.classes["@mozilla.org/preferences-service;1"].
					getService(Components.interfaces.nsIPrefBranch).getBranch('openlink.');
		var wantSubmenu = false;
		if (prefs.getPrefType("useSubmenuForLinks") == prefs.PREF_BOOL) {
			try {
				if (prefs.getBoolPref("useSubmenuForLinks")) {
					wantSubmenu = true;
				}
			} catch (Exception) {
			}
		}
		//Display menu items accordingly:
		for (var i=0; i<gOpenlinkOpenLinkMenuItems.length; i++) {
			var elementId = gOpenlinkOpenLinkMenuItems[i];
			var menuItem = document.getElementById(elementId);
			if (menuItem) {
				if ( (elementId=='openlink-openlinkinbackgroundtab' && tabsOpenInBg) ||
						(elementId=='openlink-openlinkinforegroundtab' && !tabsOpenInBg) ||
						                                                         wantSubmenu) {
					menuItem.hidden = true;
				} else {
					menuItem.hidden = !isOpenableLink;
				}
			}
		}
		//Display open link context menu accordingly:
		var openLinkListMenuItem = document.getElementById('openlink-openlinkin');
		if (openLinkListMenuItem) {
			openLinkListMenuItem.hidden = !(isOpenableLink && wantSubmenu);
		}

		//Display view image context menu if user is on a viewable image:
		var isViewableImage = gContextMenu.onImage;
		var viewImageListMenuItem = document.getElementById('openlink-viewimage');
		if (viewImageListMenuItem) {
			viewImageListMenuItem.hidden = !isViewableImage;
			//Hide the default view image item:
			var viewImageItem = document.getElementById('context-viewimage');
			if (viewImageItem) {
				viewImageItem.hidden = true;
			}
		}

		//Display view background image context menu if user is on a viewable background image:
		var isViewableBackgroundImage = gContextMenu.hasBGImage &&
		                                    !( gContextMenu.inDirList || gContextMenu.onImage ||
		                                           gContextMenu.isTextSelected || gContextMenu.onLink ||
		                                                                          gContextMenu.onTextInput );
		var viewBackgroundImageListMenuItem = document.getElementById('openlink-viewbackgroundimage');
		if (viewBackgroundImageListMenuItem) {
			viewBackgroundImageListMenuItem.hidden = !isViewableBackgroundImage;
			//Hide the default view image item:
			var viewBackgroundImageItem = document.getElementById('context-viewbgimage');
			if (viewBackgroundImageItem) {
				viewBackgroundImageItem.hidden = true;
			}
		}
	}
}


/**
 * This function is called when the open link context menu pops up.
 * It decides which open link menu elements should be shown.
 * Currently, this is everything but the inappropriate foreground/background tab element.
 */
function openlinkShowOpenLinkContextMenuItems() {
	var tabsOpenInBg = window.getBoolPref("browser.tabs.loadInBackground", false);;
	//If the open link context menu is open:
	var openLinkListMenuItem = document.getElementById('openlink-openlinkin');
	if (openLinkListMenuItem) {
		//Display menu items:
		for (var i=0; i<gOpenlinkOpenLinkMenuMenuItems.length; i++) {
			var elementId = gOpenlinkOpenLinkMenuMenuItems[i];
			var menuItem = document.getElementById(elementId);
			if (menuItem) {
				menuItem.hidden = ( (elementId=='openlink-openlinkinbackgroundtabmenu' && tabsOpenInBg) ||
				                        (elementId=='openlink-openlinkinforegroundtabmenu' && !tabsOpenInBg) );
			}
		}
	}
}


/**
 * This function is called when the view image context menu pops up.
 * It decides which view image menu elements should be shown.
 * Currently, this is everything but the inappropriate foreground/background tab element.
 */
function openlinkShowViewImageContextMenuItems() {
	var tabsOpenInBg = window.getBoolPref("browser.tabs.loadInBackground", false);;
	//If the view image context menu is open:
	var viewImageListMenuItem = document.getElementById('openlink-viewimage');
	if (viewImageListMenuItem) {
		//Display menu items:
		for (var i=0; i<gOpenlinkViewImageMenuItems.length; i++) {
			var elementId = gOpenlinkViewImageMenuItems[i];
			var menuItem = document.getElementById(elementId);
			if (menuItem) {
				menuItem.hidden = ( (elementId=='openlink-viewimageinbackgroundtab' && tabsOpenInBg) ||
				                        (elementId=='openlink-viewimageinforegroundtab' && !tabsOpenInBg) );
			}
		}
	}
}


/**
 * This function is called when the view background image context menu pops up.
 * It decides which view background image menu elements should be shown.
 * Currently, this is everything but the inappropriate foreground/background tab element.
 */
function openlinkShowViewBackgroundImageContextMenuItems() {
	var tabsOpenInBg = window.getBoolPref("browser.tabs.loadInBackground", false);;
	//If the view background image context menu is open:
	var viewBackgroundImageListMenuItem = document.getElementById('openlink-viewbackgroundimage');
	if (viewBackgroundImageListMenuItem) {
		//Display menu items:
		for (var i=0; i<gOpenlinkViewBackgroundImageMenuItems.length; i++) {
			var elementId = gOpenlinkViewBackgroundImageMenuItems[i];
			var menuItem = document.getElementById(elementId);
			if (menuItem) {
				menuItem.hidden = ( (elementId=='openlink-viewbackgroundimageinbackgroundtab' && tabsOpenInBg) ||
				                        (elementId=='openlink-viewbackgroundimageinforegroundtab' && !tabsOpenInBg) );
			}
		}
	}
}






/*
 * Derived from openNewTabWith() in utilityOverlay.js in browser.jar, simply by removing
 * all preference-checking as to whether to open in background or not, and replacing it with our own.
 * @param href The URL to open (as a string).
 * @param aDocument The document from which the URL came, or null.
 *          This is used to set the referrer header and to do a security check of whether
 *          the document as allowed to reference the URL.
 *          If null, there will be no referrer header and no security check.
 * @param aOpenInBackground true if new tab is to be opened in background, false otherwise
 */
function openlinkOpenNewTabWith(href, aDocument, aOpenInBackground) {
	if (aDocument) {
		urlSecurityCheck(href, aDocument.nodePrincipal);
	}

	// As in openNewWindowWith() defined in <install-directory>\chrome\browser.jar ->
	// content/browser/utilityOverlay.js, we want to pass the charset of the 
	// current document over to a new tab.
	var wintype = document.documentElement.getAttribute("windowtype");
	var originCharset; 
	if (wintype == "navigator:browser") { 
		originCharset = window.content.document.characterSet;
	} 

	// Open link in new tab 
	var browser = top.document.getElementById("content");
	if (browser) {
		var referrerURI = aDocument ? aDocument.documentURIObject : null;
		browser.loadOneTab(href, referrerURI, originCharset, null, aOpenInBackground);
	}
}


/*
 * Derived from openNewWindowWith() in utilityOverlay.js in browser.jar.
 * WORKS - BUT PROCEDURE ISN'T GREAT; INVOLVES REPEATEDLY FOCUSSING THE CURRENT WINDOW AFTER
 * THE NEW WINDOW HAS BEEN OPENED.
 * @param href The URL to open (as a string).
 * @param aDocument The document from which the URL came, or null.
 *          This is used to set the referrer header and to do a security check of whether
 *          the document as allowed to reference the URL.
 *          If null, there will be no referrer header and no security check.
 */
function openlinkOpenNewBackgroundWindowWith(href, aDocument) {
	if (aDocument) {
		urlSecurityCheck(href, aDocument.nodePrincipal);
	}

	// if and only if the current window is a browser window and it has a document with a character
	// set, then extract the current charset menu setting from the current document and use it to
	// initialize the new browser window...
	var charsetArg = null;
	var wintype = document.documentElement.getAttribute("windowtype");
	if (wintype == "navigator:browser") {
		charsetArg = "charset=" + window.content.document.characterSet;
	}

	var referrerURI = aDocument ? aDocument.documentURIObject : null;
	gCurrWindow = window;
	var newWindow = window.openDialog(getBrowserURL(), "_blank", "chrome,all,dialog=no", href, charsetArg, referrerURI, null);
	newWindow.addEventListener('focus', openlinkDoWindowFocus, false);
	setTimeout(function() {
	           	newWindow.removeEventListener('focus', openlinkDoWindowFocus, false);
			// Thanks t oSUN Chun-Yen for reminding me to remove the event listener once the new window had opened!

			// Unfortunately, while the current window is focused at this point, the active button on
			// the Windows Taskbar is that of the new window.  Indeed Firefox seems a little confused
			// because the following call still keeps the current window focused, and happily also
			// causes its Taskbar button to become the active one.
			gCurrWindow.blur();
	           }, 1000);
}


/**
 * Based on the procedure in openNewWindowWith() and openNewTabWith() in utilityOverlay.js in browser.jar.
 * I couldn't figure out from browser.js how normal left-clicks on links were treated, so I used the function
 * loadURI from browser.js.
 */
function openlinkOpenHereWith(href, aDocument) {
	if (aDocument) {
		urlSecurityCheck(href, aDocument.nodePrincipal);
	}
	var referrerURI = aDocument ? aDocument.documentURIObject : null;
	loadURI(href, referrerURI, null);
}


/**
 * Derived from viewImage in nsContextMenu.js in browser.jar, simply by removing
 * all preference-checking as to whether to open in background or not, and replacing it with our own.
 * @param aDocument The document from which the URL came, or null.
 *          This is used to set the referrer header and to do a security check of whether
 *          the document as allowed to reference the URL.
 *          If null, there will be no referrer header and no security check.
 * @param aIsBgImage true if object is background image, false if normal image
 * @param aTarget The string 'current' or 'tab' or 'window'
 * @param aOpenInBackground true if new tab or window is to be opened in background, false if foreground, null if unspecified
 */
function openlinkViewImageWith(aDocument, aIsBgImage, aTarget, aOpenInBackground) {
	if (!gContextMenu || !gContextMenu.browser) {
		return;
	}
	var viewURL;
	if (aIsBgImage) {
		viewURL = gContextMenu.bgImageURL;
		urlSecurityCheck(viewURL, gContextMenu.browser.contentPrincipal, Ci.nsIScriptSecurityManager.DISALLOW_SCRIPT);
	} else {
		if (gContextMenu.onCanvas) {
			viewURL = gContextMenu.target.toDataURL();
		} else {
			viewURL = gContextMenu.imageURL;
			urlSecurityCheck(viewURL, gContextMenu.browser.contentPrincipal, Ci.nsIScriptSecurityManager.DISALLOW_SCRIPT);
		}
	}

	switch(aTarget) {
		case 'current':
			//fallthrough
		case 'tab':
			if(aOpenInBackground===null) {
				///////openNewTabWith(aURL, aDocument, aPostData, aEvent, aAllowThirdPartyFixup, aReferrer)		
				window.openNewTabWith(viewURL, aDocument, null, null, null, aDocument.documentURIObject);
			} else {
				openlinkOpenNewTabWith(viewURL, aDocument, aOpenInBackground)
			}
			break;
		case 'window':
			if(aOpenInBackground) {
				openlinkOpenNewBackgroundWindowWith(viewURL, aDocument)
			} else {
				///////openNewWindowWith(aURL, aDocument, aPostData, aAllowThirdPartyFixup, aReferrer)
				window.openNewWindowWith(viewURL, aDocument, null, null, aDocument.documentURIObject);
			}
	}
}



function openlinkDoWindowFocus() {
	gCount = 0;
	openlinkFocusCurrentWindowRepeatedly();
}


function openlinkFocusCurrentWindowRepeatedly() {
	gCurrWindow.focus();
	if (gCount<gMAX) {
		++gCount;
		gCurrWindow.setTimeout('openlinkFocusCurrentWindowRepeatedly()', 20);
	}
}



function openlinkOpenLinkInBackgroundTab() {
	if (gContextMenu && gContextMenu.linkURL && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkOpenNewTabWith(gContextMenu.linkURL, gContextMenu.target.ownerDocument, true);
	}
}


function openlinkOpenLinkInForegroundTab() {
	if (gContextMenu && gContextMenu.linkURL && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkOpenNewTabWith(gContextMenu.linkURL, gContextMenu.target.ownerDocument, false);
	}
}


function openlinkOpenLinkInBackgroundWindow() {
	if (gContextMenu && gContextMenu.linkURL && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkOpenNewBackgroundWindowWith(gContextMenu.linkURL, gContextMenu.target.ownerDocument);
	}
}


function openlinkOpenLinkHere() {
	if (gContextMenu && gContextMenu.linkURL && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkOpenHereWith(gContextMenu.linkURL, gContextMenu.target.ownerDocument);
	}
}



function openlinkViewImageInNewTab() {
	if (gContextMenu && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkViewImageWith(gContextMenu.target.ownerDocument, false, 'tab', null);
	}
}


function openlinkViewImageInBackgroundTab() {
	if (gContextMenu && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkViewImageWith(gContextMenu.target.ownerDocument, false, 'tab', true);
	}
}


function openlinkViewImageInForegroundTab() {
	if (gContextMenu && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkViewImageWith(gContextMenu.target.ownerDocument, false, 'tab', false);
	}
}

function openlinkViewImageInNewWindow() {
	if (gContextMenu && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkViewImageWith(gContextMenu.target.ownerDocument, false, 'window', false);
	}
}


function openlinkViewImageInBackgroundWindow() {
	if (gContextMenu && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkViewImageWith(gContextMenu.target.ownerDocument, false, 'window', true);
	}
}


function openlinkViewImageHere() {
	if (gContextMenu && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkViewImageWith(gContextMenu.target.ownerDocument, false, 'current', null);
	}
}



function openlinkViewBackgroundImageInNewTab() {
	if (gContextMenu && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkViewImageWith(gContextMenu.target.ownerDocument, true, 'tab', null);
	}
}


function openlinkViewBackgroundImageInBackgroundTab() {
	if (gContextMenu && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkViewImageWith(gContextMenu.target.ownerDocument, true, 'tab', true);
	}
}


function openlinkViewBackgroundImageInForegroundTab() {
	if (gContextMenu && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkViewImageWith(gContextMenu.target.ownerDocument, true, 'tab', false);
	}
}

function openlinkViewBackgroundImageInNewWindow() {
	if (gContextMenu && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkViewImageWith(gContextMenu.target.ownerDocument, true, 'window', false);
	}
}


function openlinkViewBackgroundImageInBackgroundWindow() {
	if (gContextMenu && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkViewImageWith(gContextMenu.target.ownerDocument, true, 'window', true);
	}
}


function openlinkViewBackgroundImageHere() {
	if (gContextMenu && gContextMenu.target && gContextMenu.target.ownerDocument) {
		openlinkViewImageWith(gContextMenu.target.ownerDocument, true, 'current', null);
	}
}
